"""
File with methods used to anonymize, clean and order the analysis data.
Because the initial raw data contained Prolific IDs, that data is not made public.

The available raw data consists of:
- cleaned_database_data.csv

The pre-processed data consists of:
- preprocessed_pre_screening_data.csv
- preprocessed_database_data.csv
- preprocessed_post_questionnaire_data.csv
- preprocessed_demographic_data.csv

Author: Martin Dierikx
Date: November 2023
"""


def processPreScreening(pre_screening_data_file, participants, processed_pre_screening_data_file):
    """
    Method used to pre-process the pre-screeening questionnaire data.
    It ananomizes the participant ids and removes the data that cannot be published.
    Because the raw data contained prolific ids and other data we were not allowed to publish, that data is not published.

    Args:
        pre_screening_data_file: path to file containing pre-screening data.
        participants: list of ids of all participants of the study.
        processed_pre_screening_data_file: path to file to store the pre-processed data.

    Returns: Creates the preprocessed_pre_screening_data.csv file.

    """
    # The unprocessed pre-screeening data had the following fields:
    # - Startdate of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Enddate of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Duration of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Recorded date of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Response id of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Consent from participant to participate in the study
    # - - Since all participants had to give consent to participate, this column contained no relevant data and was removed.
    # - Prolific id of the participant
    # - - Data we anonimized to remove personal information.
    # - Fluent languages of the participant
    # - - Data we removed because we did not get permission to publish it.
    # - Engagement in physical exercise per week of the participant
    # - Average steps per day of the past week per of the participant
    # - Way of tracking steps of the participant
    # - Free text response in case the answer to the previous question was 'other'
    # - Whether the participant was contemplating or preparing to become more physically active
    # - Godin leisure time questionnaire question 1
    # - Godin leisure time questionnaire question 2
    # - Godin leisure time questionnaire question 3
    # - Prolific id of the participant
    # - - Data we removed because we already had it in a previous column.

    pre_screening_data = []
    for d in open(pre_screening_data_file):
        data = d.split(",")
        # If the length of the first cell is longer than 13, it means it is not a header cell and it thus contains data of a person.
        if len(data[0]) >= 14:
            # Anonymize the id of the participant
            data[6] = "p" + str(participants.index(data[6]) + 1)
            pre_screening_data.append(data)

    # The processed pre-screening data has only the following fields:
    # - Anonymized id of the participant
    # - Engagement in physical exercise per week of the participant
    # - Average steps per day of the past week per of the participant
    # - Way of tracking steps of the participant
    # - Free text response in case the answer to the previous question was 'other'
    # - Whether the participant was contemplating or preparing to become more physically active
    # - Godin leisure time questionnaire question 1
    # - Godin leisure time questionnaire question 2
    # - Godin leisure time questionnaire question 3

    f = open(processed_pre_screening_data_file, "w")
    f.write(
        f"ID,Exercise_per_week,Steps_on_average,Way_of_tracking_steps,other,TTM_stage,Godin1,Godin2,Godin3\n")
    f.close()

    for data in pre_screening_data:
        f = open(processed_pre_screening_data_file, "a")
        f.write(
            f"{data[6]},{data[8]},{data[9]},{data[10]},{data[11]},{data[12]},{data[13]},{data[14]},{data[15]}")
        f.close()


def anonimizeDatabaseData(database_data_file, participants, cleaned_database_data_file):
    """
    Method used to anonimize and clean the database file.
    Because the raw data contained prolific ids and other data we were not allowed to publish, that data is not published.

    Args:
        database_data_file: path to the file containing the database data.
        participants: list of ids of all participants of the study.

    Returns: Creates the cleaned_database_data.csv file
    """
    # The uncleaned database data had the following fields:
    # - Prolific id of the participant
    # - - Data we anonimized to remove personal information.
    # - Session number of the datapoint
    # - Type of information of the datapoint (e.g. previous activity, mood, self-motivation, etc.)
    # - Actual information of the datapoint
    # - Time of data registration
    # - - Data we removed because we did not get permission to publish it.

    p_ids = []
    session_numbers = []
    response_types = []
    response_values = []
    for d in open(database_data_file):
        data = d.split(",")
        # Get the participant id of the data point
        p_id = data[0].strip("\"")

        # Check if the data belongs to a person of whom we are allowed to use the data.
        if p_id in participants:
            # Anonimize the data
            p_id = "p" + str(participants.index(p_id) + 1)
            p_ids.append(p_id)
            session_numbers.append(data[1])
            response_types.append(data[2])
            if data[2] == "prev_activity":
                response_values.append(
                    data[3] + "," + data[4] + "," + data[5] + "," + data[6] + "," + data[7] + "," + data[8] + "," +
                    data[9] + "," + data[10] + "," + data[11])
            else:
                response_values.append(data[3])

    # The cleaned database data had the following fields:
    # - Anonimized id of the participant
    # - Session number of the datapoint
    # - Type of information of the datapoint (e.g. previous activity, mood, self-motivation, etc.)
    # - Actual information of the datapoint

    f = open(cleaned_database_data_file, "w")
    f.write(f"ID,Session number,Response type,Response value\n")
    for i in range(len(p_ids)):
        f.write(f"{p_ids[i]},{session_numbers[i]},{response_types[i]},{response_values[i]}\n")
    f.close()


def processPostQuestionnaire(post_data_file, participants, processed_post_data_file):
    """
    Method used to pre-process the pre-screeening questionnaire data.
    It ananomizes the participant ids and removes the data that cannot be published.
    Because the raw data contained prolific ids and other data we were not allowed to publish, that data is not published.

    Args:
        post_data_file: path to file containing post data.
        participants: list of ids of all participants of the study.
        processed_post_data_file: path to file to store the processed post data.

    Returns: Creates the preprocessed_post_questionnaire_data.csv file.

    """
    # The unprocessed post-questionnaire data had the following fields:
    # - Startdate of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Enddate of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Duration of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Recorded date of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Response id of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Prolific id of the participant
    # - - Data we anonimized to remove personal data.
    # - Number of steps of the previous day of the participant
    # - Perceived goal achievability of the goal of the previous day of the participant
    # - Self-efficacy accuracy of the self-efficacy of the previous day of the participant
    # - ASAQ short version question 1
    # - ASAQ short version question 2
    # - ASAQ short version question 3
    # - ASAQ short version question 4
    # - ASAQ short version question 5
    # - ASAQ short version question 6
    # - ASAQ short version question 7
    # - ASAQ short version question 8
    # - Attention check 1
    # - - Data we removed because it was not relevant and the answers of all the participants were the same.
    # - ASAQ short version question 9
    # - ASAQ short version question 10
    # - ASAQ short version question 11
    # - ASAQ short version question 12
    # - ASAQ short version question 13
    # - ASAQ short version question 14
    # - ASAQ short version question 15
    # - ASAQ short version question 16
    # - ASAQ short version question 17
    # - Attention check 2
    # - - Data we removed because it was not relevant and the answers of all the participants were the same.
    # - ASAQ short version question 18
    # - ASAQ short version question 19
    # - ASAQ short version question 20
    # - ASAQ short version question 21
    # - ASAQ short version question 22
    # - ASAQ short version question 23
    # - ASAQ short version question 24
    # - Perceived personalization of the proposed goals of the participant
    # - Why the participant felt that way
    # - Perceived goal difficulty of the goals of the participant
    # - Why the participant felt that way
    # - Most motivating part of the sessions according to the participant
    # - Prolific id of the participant
    # - - Data we removed because we already had it in a previous column.

    post_questionnaire_data = []
    for d in open(post_data_file):
        data = d.split(",")
        # If the length of the first cell is longer than 13, it means it is not a header cell and it thus contains data of a person.
        if len(data[0]) >= 14:
            # Anonymize the id of the participant
            data[5] = "p" + str(participants.index(data[5]) + 1)
            post_questionnaire_data.append(data)

    # The processed post-questionnaire data has the following fields:
    # - Anonymized id of the participant
    # - Number of steps of yesterday of the participant
    # - Perceived goal achievability of the goal of yesterday of the participant
    # - Self-efficacy accuracy of the self-efficacy of yesterday of the participant
    # - ASAQ short version question 1
    # - ASAQ short version question 2
    # - ASAQ short version question 3
    # - ASAQ short version question 4
    # - ASAQ short version question 5
    # - ASAQ short version question 6
    # - ASAQ short version question 7
    # - ASAQ short version question 8
    # - ASAQ short version question 9
    # - ASAQ short version question 10
    # - ASAQ short version question 11
    # - ASAQ short version question 12
    # - ASAQ short version question 13
    # - ASAQ short version question 14
    # - ASAQ short version question 15
    # - ASAQ short version question 16
    # - ASAQ short version question 17
    # - ASAQ short version question 18
    # - ASAQ short version question 19
    # - ASAQ short version question 20
    # - ASAQ short version question 21
    # - ASAQ short version question 22
    # - ASAQ short version question 23
    # - ASAQ short version question 24
    # - Perceived personalization of the proposed goals of the participant
    # - Why the participant felt that way
    # - Perceived goal difficulty of the goals of the participant
    # - Why the participant felt that way
    # - Most motivating part of the sessions according to the participant

    f = open(processed_post_data_file, "w")
    f.write(
        f"ID,Steps,Goal_achievability,Confidence_accuracy,ASA1,ASA2,ASA3,ASA4,ASA5,ASA6,ASA7,ASA8,ASA9,ASA10,ASA11,ASA12,ASA13,ASA14,ASA15,ASA16,ASA17,ASA18,ASA19,ASA20,ASA21,ASA22,ASA23,ASA24,Goal_personalization,Why,Goal_difficulty,Why,Motivation\n")
    f.close()

    for data in post_questionnaire_data:
        f = open(processed_post_data_file, "a")
        f.write(
            f"{data[5]},{data[6]},{data[7]},{data[8]},{data[9]},{data[10]},{data[11]},{data[12]},{data[13]},{data[14]},{data[15]},{data[16]},{data[18]},{data[19]},{data[20]},{data[21]},{data[22]},{data[23]},{data[24]},{data[25]},{data[26]},{data[28]},{data[29]},{data[30]},{data[31]},{data[32]},{data[33]},{data[34]},{data[35]},{data[36]},{data[37]},{data[38]},{data[39]}")
        f.close()


def processDemographics(demographic_data_file, participants, processed_demographic_data_file):
    """
    Method used to pre-process the demographic data from Prolific.
    It ananomizes the participant ids and removes the data that cannot be published.
    Because the raw data contained prolific ids and other data we were not allowed to publish, that data is not published.

    Args:
        demographic_data_file: path to file containing demographic data.
        participants: list of ids of all participants of the study.
        processed_demographic_data_file: path to file to store processed demographic data.

    Returns: Creates the preprocessed_demographic_data.csv file.

    """
    # The unprocessed demographic data had the following fields:
    # - Submission id of the submission
    # - - Data we removed because we did not get permission to publish it.
    # - Prolific id of the participant
    # - - Data we anonimized to remove personal data.
    # - Start date of submission
    # - - Data we removed because we did not get permission to publish it.
    # - End date of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Review date of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Archived date of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Duration of submission
    # - - Data we removed because we did not get permission to publish it.
    # - Completion code of the submission
    # - - Data we removed because we did not get permission to publish it.
    # - Total approved Prolific submissions of the participant
    # - Gender of the participant
    # - Fluent languages of the participant
    # - - Data we removed because we did not get permission to publish it.
    # - Highest education level of the participant
    # - Engagement in physical exercise per week of the participant
    # - Age of the participant
    # - Country of residence of the participant
    # - - Data we removed because we did not get permission to publish it.

    participants_data = []
    for d in open(demographic_data_file):
        data = d.split(",")
        # If the length of the first cell is longer than 23, it means it is not a header cell and it thus contains data of a person.
        if len(data[0]) >= 24:
            # Anonymize the id of the participant
            data[1] = "p" + str(participants.index(data[1]) + 1)
            participants_data.append(data)

    # The processed demographic data had the following fields:
    # - Anonymized id of the participant
    # - Total approved Prolific submissions of the participant
    # - Gender of the participant
    # - Highest education level of the participant
    # - Engagement in physical exercise per week of the participant
    # - Age of the participant

    f = open(processed_demographic_data_file, "w")
    f.write(
        f"ID,Approvals,Gender,Education_level,Exercise,Age\n")
    f.close()

    for data in participants_data:
        f = open(processed_demographic_data_file, "a")
        f.write(f"{data[1]},{data[8]},{data[9]},{data[11]},{data[12]},{data[13]}")
        f.close()
